;/*
; *****************************************************************************
; * Copyright by ams AG                                                       *
; * All rights are reserved.                                                  *
; *                                                                           *
; * IMPORTANT - PLEASE READ CAREFULLY BEFORE COPYING, INSTALLING OR USING     *
; * THE SOFTWARE.                                                             *
; *                                                                           *
; * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS       *
; * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT         *
; * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS         *
; * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT  *
; * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,     *
; * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT          *
; * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,     *
; * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY     *
; * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT       *
; * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE     *
; * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.      *
; *****************************************************************************
; */

;---- File    	:GP30Y_A1.D2.11.04.asm
;---- Created 	: 2016/10/24
;---- last change: 2016/10/26
;---- Authors	: VK, HG, SK
;---- Company  	: acam-messelectronic gmbh
;---- Project  	: GP30Y
;----
;---- Contents 	: Firmware with flow calculation (defined as of 20.05.2014) and 
;----			  4-wire temperature measurement. This firmware uses all 
;----			  the currently implemented ROM routines.
;---- Based on vA1.A1.22.01.  This is the first release for mass production
;---- It considers all bug fixes known to date. 
;---- It features extensive error handling and FHL regulation routines
;---- Build A1.A2.11.02 has only one change: the production trimbits set to 0x13E4
;---- Build A1.A2.11.03 has command mcten removed to avoid (improbable) data collisions. Minor bugfixes in filter initialization 
;---- Build A1.A2.11.04 contains a bugfix against wrong initialization after power up; COM_REG is being actively cleared; all changes are in the open part
;---- Note that this version is NOT down compatible with former open firmware versions A1.D1.XX.XX

;================================== Info about file headers ==================================
; The file headers contain a short description and a (hopefully) complete description of in and output.
; "(total) unused" refers to registers X,Y,Z and R and includes all internal subroutine calls - meaning the register is unchanged and never used in the whole process until jsubret.
; Temporary RAM are RAM cells which are used and changed, but need no initialisation and need not be kept after the call
; Permanent RAM are RAM and FWD cells which either need to be initialized and/or contain results for further processing after jsubret
; 	Note that some routines use relative addresses, such that the permanent RAM address depends on the input parameter.
;	Note: RAM used in subroutines is not listed here; note also that permanent RAM in a subroutine may be only temporary for the calling routine.
; Routines used summarizes calls within the routine, but calls in subroutines are not listed here
;=============================================================================================

#include "GP30Y_A1.D2.11.04.h"					; header file containing memory address and constants definitions
#include "GP30Y_ROM_A1.common.h"				; Definition of ROM routine start addresses - commons
;#include "GP30Y_UPD_A1.E2.11.01.h"				; Updates of ROM routine start addresses - commons - included in this file, too
#include "GP30Y_REG_A1.2.h"						; Definition of Register addresses, bit positions and register actions


CONST FW_ROMVERSION_REV			0xA1			;
CONST FW_VERSION_NUM			0xD20000		;
CONST FW_VERSION_MAJ			0x001000		;
CONST FW_VERSION_MIN			0x000100		;
CONST FW_VERSION_BLD			0x000004		;
CONST FW_VERSION				FW_VERSION_NUM + FW_VERSION_MAJ + FW_VERSION_MIN + FW_VERSION_BLD  ;

; The following org command is needed at the start of the code
	org		0
;========================================================================================
;================================== Check of CPU Requests - Main acam Firmware routine
;========================================================================================
; This routine checks the status flags for activity requests and calls the according routines	
; Inputs 		:	Status flags in SRR_FEP_STF and SHR_CPU_REQ
; Output 		:	all processing done and status flags cleared
; (total)unused	: (all used)
; Temporary RAM : -	
; Permanent RAM : RAM_R_FW_STATUS	
; Routines used : ROM_FWI, ROM_EH, MK_PP, MK_GPH, ROM_CPU_CHK    
;======================================================================================== 

;############# LEAVE THE FOLLOWING 9 LINES ON TOP OF THE CODE WITHOUT ANY CHANGE ###########################
MK_CPU_REQ:
	nop								;--   	  
;-------------------- The following routine does any necessary initialisation, function and security check as well as regularly restoring permanent memory cells
; 					  This routine must be called before any other activity, in particular before any ROM call; else some security measures are not operational
; 					  Note: This routine enforces some configurations regularly; it must be modified if this is undesired. 
; 					  Enforced configurations: measurement cycle timer enabled, Watchdog enabled, checksum timer every hour, restore of configurations from NVRAM with checksum timer, GPIO6 configured to output
	goto		MK_CHK_AND_RESUME1   ; This routine may start over at ROM_CPU_CHK or even stop code execution; it is called by goto to avoid confusion with the call stack 
   MK_CHK_AND_RESUME_RETURN:			
;############# LEAVE THE PRECEEDING 9 LINES ON TOP OF THE CODE WITHOUT ANY CHANGE  ###########################



; ------------------- Always call error handling
	;ramadr 	SHR_CPU_REQ			;-- Set RAM Address to SHR_CPU_REQ
	;skipBitC 	r, BNR_EH, 1		;-- Check Error Handling Flag: this flag shall not be used since it triggers non-synchronous events which confuses the task sequencer
	jsub 		MK_EH				;-- Always jump to User Error Handling Subroutine; now modified ROM routine; change of A1.A1.22.01
	
	
; -------------------Here comes the main call for post processing		
	ramadr 		SHR_CPU_REQ			;-- Set RAM Address to SHR_CPU_REQ	
	skipBitC 	r, BNR_PP, 3		;-- Check User Memory Post Processing Flag
	  jsub 		MK_PP				;-- Jump to User Post Processing
	  ; in case a communication request was set, the FW has requested an interrupt at next IDLE state
	  ; clear the communication request in any case to prevent more subsequent interrupts
	  ramadr	SHR_EXC
	  bitset	r,BNR_COM_REQ_CLR	; change of A1.A2.11.04
	

; -------------------- General purpose routine: is only called after a general purpose request.
; replace register SHR_CPU_REQ and bit BNR_GPH with any other desired flag to have your routine called as desire
; remember to reset the same flag bit inside the routine MK_GPH. Else the routine will be called again after next measurement automatically
	ramadr 		SHR_CPU_REQ			;-- Set RAM Address to SHR_CPU_REQ; update of vA1.A1.12.01	
	skipBitC 	r, BNR_GPH, 1		;-- Check General Purpose Flag
	  jsub 		MK_GPH				;-- Jump to User General Purpose Handling
	  
; -------------------- End of the main program
	ramadr 		SHR_CPU_REQ			;-- Set RAM Address to SHR_CPU_REQ
	skipBitS	r,BNR_BLD_EXC,2		;-- check if BNR_BLD_EXC or BNR_CHKSUM may still be set; then skip stop; change of A1.A1.22.01
	skipBitS	r,BNR_CHKSUM,1

	  stop							; end execution without looking back; this avoids infinite loops
	goto 		ROM_CPU_CHK			; jump back into loop only if BNR_BLD_EXC or BNR_CHKSUM are set
	
;###########################################################################################
;      End of Main Program MK_CPU_REQ
;###########################################################################################
	

;========================================================================================
;================================== General Purpose Handling 
;========================================================================================
; The subroutine is not active yet and just clears the GPH flag
; Inputs 		:	-
; Output 		:	-
; (total)unused	: (all used)
; Temporary RAM : -	
; Permanent RAM : -	
; Routines used : - 
;======================================================================================== 

MK_GPH:
	;######### Place your code here for general purpose handling #####
	nop								;-- 
;	nop								;-- 
;	nop								;-- 
;	nop								;-- 
	;#################################################################

MK_GPH_END:	
;---- clear flag bit which caused the call, to signal that the call was processed
;---- if a different flag was used to call this routine, also remember adapt the code by clearing this different flag
	ramadr 		SHR_CPU_REQ			;-- Set RAM Address to SHR_CPU_REQ
    bitclr		r, BNR_GPH			;-- Clear General Purpose Handling Flag (or whatever other flag was used for calling)

	jsubret
	
;###########################################################################################
;      End of MK_GPH
;###########################################################################################



; place your own code here !!!!!




;###########################################################################################
; leave the following lines at the end of your modified code !


;========================================================================================
;================================== MK_CHK_AND_RESUME1 ; change of A1.A2.11.04
;========================================================================================
; The subroutine fixes an initialisation bug of MK_CHK_AND_RESUME, then jumps with goto to MK_CHK_AND_RESUME
; Note that MK_CHK_AND_RESUME may start over at ROM_CPU_CHK or even stop code execution; 
; As MK_CHK_AND_RESUME1 it is called by goto to avoid confusion with the call stack 
;======================================================================================== 
; This org statement moves the routine to the end of the code	
; DON'T MOVE THIS ROUTINE UP, ELSE IT MAY BE COMPILED WITH A RELATIVE GOTO - THIS WOULD CHANGE THE FIX RETURN ADDRESS OF MK_CHK_AND_RESUME_RETURN 
	org 	OPEN_REV_ADDR - 10
MK_CHK_AND_RESUME1:
;-----the next command block initializes the flag "BNR_ROM_TEMP_ERR" at chip start before calling "MK_CHK_AND_RESUME"  
	ramadr		SHR_CPU_REQ				
	skipBitC	r,BNR_FWI,2				; clear flag "BNR_ROM_TEMP_ERR" only if firmware init is due (at first cycle)
	  ramadr	RAM_R_FW_ERR_FLAGS
	  bitClr	r,BNR_ROM_TEMP_ERR
;-------------------- The following routine does any necessary initialisation, function and security check as well as regularly restoring permanent memory cells
; 					  This routine must be called before any other activity, in particular before any ROM call; else some security measures are not operational
; 					  Note: This routine enforces some configurations regularly; it must be modified if this is undesired. 
; 					  Enforced configurations: measurement cycle timer enabled, Watchdog enabled, checksum timer every hour, restore of configurations from NVRAM with checksum timer, GPIO6 configured to output
	goto		MK_CHK_AND_RESUME   ; This routine may start over at ROM_CPU_CHK or even stop code execution; it is called by goto to avoid confusion with the call stack 
;	jsubret - never get here after goto
	
;========================================================================================
;================================== End of MK_CHK_AND_RESUME1 
;========================================================================================

;========================================================================================
;================================== Write Version number at the end of the readable code 
;========================================================================================	
	
	org 	OPEN_REV_ADDR	
	stop							; stop to avoid running into version number
	equal1	0x00					; This byte is used in acam code to achieve identical checksums for A and D code type
	equal1	FW_ROMVERSION_REV	;
	equal	FW_VERSION			; Defined at the beginning of this file
	

; fill remaining file with zeros; 
; the remaining part of the original code is confidential and read/write protected

	org 4095
	nop

;###########################################################################################
;      End of acam FW
;###########################################################################################

